#! /usr/bin/env python
#
# -*- coding: latin1 -*-
"""
Generate an extended pedigree for the specified animal.

Used as an executable, Generates a .csv file that can be
imported into a spreadsheet.

USAGE:
gen_pedigree [database file] <animal name>


Copyright (C) 2007 - 2021 Ben E. Schmidtke Jr.

Revised 11/01/11  Add support for direct spreadsheet generation


Revised 01/01/09  Add windowed environment support

Revised 12/31/07  Replace empty color field with dash "-".
                  Print "Service Unavailable" when received from web site.
                  Added Date Of Birth, Left and Right tattoos.
                  
Revised 05/20/07  Key search on registration numbers not names.

Revised 04/26/07  Truncate name comparisons at any opening parenthesis.
                  Add more error checking.

"""

import sys
import tkMessageBox
from Tkinter  import *

"""
 Simple dialog support
"""

class Dialog(Toplevel):
    """
    Class to open dialog
    """
    def __init__(self, parent, title = None):

        """
        Initialize a dialog.

        Arguments:

            parent -- a parent window (the application window)

            title -- the dialog title
        """
        
        Toplevel.__init__(self, parent)
        self.transient(parent)

        if title:
            self.title(title)

        self.parent = parent

        self.result = None

        body = Frame(self)
        self.initial_focus = self.body(body)
        body.pack(padx=5, pady=5)

        self.buttonbox()

        self.wait_visibility() # window needs to be visible for the grab
        self.grab_set()

        if not self.initial_focus:
            self.initial_focus = self

        self.protocol("WM_DELETE_WINDOW", self.cancel)

        if self.parent is not None:
            self.geometry("+%d+%d" % (parent.winfo_rootx()+50,
                                      parent.winfo_rooty()+50))

        self.initial_focus.focus_set()

        self.wait_window(self)

    def destroy(self):
        """
        Destroy the window
        """
        self.initial_focus = None
        Toplevel.destroy(self)

    #
    # construction hooks

    def body(self, master):
        """
        create dialog body.

        return widget that should have initial focus.
        This method should be overridden, and is called
        by the __init__ method.
        """
        pass


    def buttonbox(self):
        """
        add standard button box.

        override if you do not want the standard buttons
        """

        box = Frame(self)

        w = Button(box, text="OK", width=10, command=self.ok, default=ACTIVE)
        w.pack(side=LEFT, padx=5, pady=5)
        w = Button(box, text="Cancel", width=10, command=self.cancel)
        w.pack(side=LEFT, padx=5, pady=5)

        self.bind("<Return>", self.ok)
        self.bind("<Escape>", self.cancel)

        box.pack()

    #
    # standard button semantics

    def ok(self, event=None):

        if not self.validate():
            self.initial_focus.focus_set() # put focus back
            return

        self.withdraw()
        self.update_idletasks()

        self.apply()

        self.cancel()

    def cancel(self, event=None):

        # put focus back to the parent window
        if self.parent is not None:
            self.parent.focus_set()
        self.destroy()

    #
    # command hooks

    def validate(self):
        """
        validate the data

        This method is called automatically to validate the data before the
        dialog is destroyed. By default, it always validates OK.
        """

        return 1 # override

    def apply(self):
        """
        process the data

        This method is called automatically to process the data, *after*
        the dialog is destroyed. By default, it does nothing.
        """

        pass # override


# --------------------------------------------------------------------
# convenience dialogues

class _QueryDialog(Dialog):

    def __init__(self, title, prompt,
                 initialvalue=None,
                 parent = None):

        if not parent:
            import Tkinter
            parent = Tkinter._default_root

        self.prompt   = prompt

        self.initialvalue = initialvalue

        Dialog.__init__(self, parent, title)

    def destroy(self):
        self.entry = None
        Dialog.destroy(self)

    def body(self, master):

        w = Label(master, text=self.prompt, justify=LEFT)
        w.grid(row=0, columnspan=32, padx=5, sticky=W)

        self.entry = Entry(master, name="entry")
        self.entry.grid(row=1, columnspan=32, padx=5, sticky=W+E)

        if self.initialvalue:
            self.entry.insert(0, self.initialvalue)
            self.entry.select_range(0, END)

        return self.entry

    def validate(self):

        import tkMessageBox

        try:
            result = self.getresult()
        except ValueError:
            tkMessageBox.showwarning(
                "Illegal value",
                self.errormessage + "\nPlease try again",
                parent = self
            )
            return 0

        self.result = result

        return 1


class _QueryString(_QueryDialog):
    def __init__(self, *args, **kw):
        if kw.has_key("show"):
            self.__show = kw["show"]
            del kw["show"]
        else:
            self.__show = None
        _QueryDialog.__init__(self, *args, **kw)

    def body(self, master):
        entry = _QueryDialog.body(self, master)
        if self.__show is not None:
            entry.configure(show=self.__show)

        return entry

    def getresult(self):
        return self.entry.get()


class displayProgress(object):
    def __init__(self):
        "Initialize this class"
        import Tkinter
        
        self.root = Tk()
        self.sb   = Tkinter.Scrollbar()
        self.lb   = Tkinter.Listbox(width = 60)

        self.sb.pack(side=Tkinter.RIGHT, fill=Tkinter.Y)
        self.lb.pack(side=Tkinter.LEFT, fill=Tkinter.Y)

        self.sb.config(command=self.lb.yview)
        self.lb.config(yscrollcommand=self.sb.set)
        self.root.update()
        
    def addLine(self, listItem):
        "Add a line to the listbox"
        
        self.lb.insert(END, listItem)
        self.root.update()

    def destroy(self):
        self.entry = None
        object.destroy(self)

class displayDbUpdate(object):
    def __init__(self):
        "Initialize this class"
        import Tkinter
        
        self.root = Tk()
        self.sb   = Tkinter.Scrollbar()
        self.lb   = Tkinter.Listbox(width = 60)

        self.sb.pack(side=Tkinter.RIGHT, fill=Tkinter.Y)
        self.lb.pack(side=Tkinter.LEFT, fill=Tkinter.Y)

        self.sb.config(command=self.lb.yview)
        self.lb.config(yscrollcommand=self.sb.set)
        self.root.update()
        
    def addLine(self, listItem):
        "Add a line to the listbox"
        
        self.lb.insert(END, listItem)
        self.root.update()

    def destroy(self):
        self.entry = None
        object.destroy(self)




class ReadCSVError(Exception):
    pass

row=[]
complete = 1
primary_row = []
secondary_row = []
tertiary_row = []


        
def find_number(masterdb, l):
    foundit = 0

    for a in masterdb:
        dbNumber = a[0]

        if l == dbNumber:
            foundit = 1
            break
        else:
            dbNumber = a[1]
        
            if l == dbNumber:
                foundit = 1
                break

    return foundit
    
# find_sire_or_dam
# Look for name in database file.
def find_sire_or_dam(masterdb, l):
    entry = ['-', '-', '-', '-', '-', '-', '-', '-', '-','-', '-', '-']
    foundit = 0
    
    if l != '':
        foundit = find_number(masterdb, l)

    if foundit == 1:
    
        for a in masterdb:
            dbNumber = a[0]

            if l == dbNumber:
                entry = a
                break

            else:
                dbNumber = a[1]

                if l == dbNumber:
                    entry = a
                    break

    return entry


# build_animal_entry
# Creates the three lines worth of information for an animal.
# These lines are suitable for including in the CSV file
# for the pedigree.

def build_animal_entry(animal, sire_dam_flag):
    global primary_row, secondary_row, tertiary_row

    if animal[6].strip('"') == "Bull":
        primary_row = ["Sire: "]
    elif animal[6].strip('"') == "Cow":
        primary_row = [" Dam: "]
    elif sire_dam_flag == 1:
        primary_row = ["Sire: "]
    else:
        primary_row = [" Dam: "]
        
    secondary_row = ["Reg. #:"]
    secondary_row.append("Color:")


    primary_row.append(animal[2])

    if animal[0].strip('"') == "":
        secondary_row.insert(1, "-")
    else:
        secondary_row.insert(1, animal[0])
        
    if animal[7].strip('"') == "":
        secondary_row.append("-")
    else:
        secondary_row.append(animal[7])
    
    tertiary_row = [""]

    if animal[1].strip('"') == "":
        tertiary_row.append("-")
    else:
        tertiary_row.append(animal[1])

# get_animal_dob
# Returns the Dat Of Birth for the specified animal.

def get_animal_dob(animal):

    return animal[3]

# get_animal_r_tattoo
# Returns the Right ear tattoo for the specified animal.

def get_animal_r_tattoo(animal):

    return animal[5]

# get_animal_l_tattoo
# Returns the Left ear tattoo for the specified animal.

def get_animal_l_tattoo(animal):

    return animal[4]

# get_animal_sex
# Returns the sex for the specified animal.

def get_animal_sex(animal):

    return animal[6].strip('"')


def askstring(title, prompt, **kw):
    """
    get a string from the user

    Arguments:

        title -- the dialog title
        prompt -- the label text
        **kw -- see SimpleDialog class

    Return value is a string
    """
    d = _QueryString(title, prompt, **kw)
    return d.result

def getAnimalName():

    root = Tk()
    root.update()

    inputStr = askstring("gen_pedigree", "Enter the animal's name")

    root.destroy()

    return inputStr



def findAnimalInDatabase(csvFile, subString):
    """
    Case insensitive search for the animal in question
    """
    
    haveOne = 0
    
    for x in range(fLen):
        l = csvFile[x]

        name = l[2]

        foundIt = name.upper().find(subString.upper())

        if foundIt >= 0:
            haveOne += 1
            
    return haveOne


def showAnimalMatches(dwp, csvFile, subString):
    """
    Print all animals that match
    """
    
    for x in range(fLen):
        l = csv[x]

        regNo=l[0]
        fRegNo=l[1]
        name = l[2]

        foundIt = name.upper().find(subString.upper())

        if foundIt >= 0:

            # Print the name out
            if dpw == None:
                print "%5s  %7s   %s" % (regNo, fRegNo, name)
            else:
                displayProgress.addLine(dpw, "%8s        %7s      %s" % (regNo, fRegNo, name))



#------------------------------------------------------------------------

class CSV_SpreadData(list):
    """
    csv_spreadData: a=csv_spreadData("file")

    the class csv_spreadData which is simply a list of lists initialized with
    the contents of the spreadsheet stored in a .csv file (optionally passed as
    an argument). If no file name is given, it is defaulted to master_db.csv.

    """

    def __init__(self, fname):
        global row, primary_row, secondary_row, tertiary_row
        row=[]
        fLen = 0
        list.__init__(self)

if __name__=="__main__":

    fname = "master_db.csv" # default
    
    # Attempt to determine whether windowed or command line environment
    noArgs = 0
    temp = 0
    
    if len(sys.argv)==3:
        fname = sys.argv[1]
        nameIn =  sys.argv[2]
    elif len(sys.argv)==2:
        nameIn =  sys.argv[1]
    else:
        nameIn = getAnimalName()
        noArgs = 1
        
    if nameIn == None:
        if noArgs == 0:
            print >> sys.stderr, "Usage: %s [database file] <animal_name>" % sys.argv[0]
        else:
            tkMessageBox.showwarning("Error", "Usage: %s [database file] <animal name>" % sys.argv[0])
            
        sys.exit(1)


    
    # Read the database file contents.
    try:
        f = open(fname, 'r')
        fLen = len(f.readlines())
        f.seek(0)

        s = f.read().strip('"')
            
        f.close()

    except(IOError, OSError):
        if noArgs == 0:
            print fname + " not found"
        else:
            tkMessageBox.showerror("Error", fname + " not found.")
            
        sys.exit(1)

    csv=[]
    raw = []
    raw1 = []
    dataCompleteList = []

    # Remove all quotes and newlines and create a list of the lines in the
    # input file
    raw1 = s.replace('"',"")

    raw = raw1.splitlines(0)

    for x in range(fLen):
        csv.append(raw[x].split(','))
                   
    # First, locate the animal in question in the database.

    haveValidEntries = findAnimalInDatabase(csv, nameIn)

    if haveValidEntries > 1:
        if noArgs == 0:
            print "More than one animal in the databsae have %s in their name.\n" % nameIn
            print "Please refine your search\n"
        else:
            dpw = displayProgress() # Create the window for further messages
            showAnimalMatches(dpw, csv, nameIn)
            tkMessageBox.showwarning("Please Refine Search", "More than one match in the database for \"%s\"." % nameIn)

        sys.exit(1)
    
    
    complete = 0
    
    for x in range(fLen):
        l = csv[x]

        regNo=l[0]
        fRegNo=l[1]
        name = l[2]

        if regNo == nameIn:
            nameIn = name
            complete = 1
            break

        elif fRegNo == nameIn:
            nameIn = name
            complete = 1
            break

        else:
            temp = name.upper().find(nameIn.upper())

            if temp >= 0:
                complete = 1
                break

    if complete == 0:
        print >> sys.stderr, nameIn + " not in data base. Check spelling."
        tkMessageBox.showwarning("Error", nameIn + " not in data base. Check spelling.")
        sys.exit(1)

    # Some helpful initializers
    dashedList = ['-','-','-','-','-','-','-','-','-','-','-','-']
    Sire = 1
    Dam  = 0
    foundMatch = 0
    
    # Initialize our lists
    sire1 = sire2 = sire3 = sire4 = sire5 = sire6 = sire7 = sire8 = dashedList
    sire9 = sire10 = sire11 = sire12 = sire13 = sire14 = sire15 = dashedList
    dam1 = dam2 = dam3 = dam4 = dam5 = dam6 = dam7 = dam8 = dashedList
    dam9 = dam10 = dam11 = dam12 = dam13 = dam14 = dam15 = dashedList

    lines = list()

    # Create a list of flags to indicate whether we have completed 
    # this part of the tree.
    for x in range(31):
        dataCompleteList.append(0)
    
    # Create an empty list of the lines to be displayed in the pedigree.
    for x in range(63):
        lines.append([''])

    child = l

    x = len(child)
    
    # Locate sire for input animal.
    foundMatch = 0
        
    if x > 10 and child[8] != '':

        for m in csv:
            if m[0] == child[8] or m[1] == child[8]:
                sire1 = find_sire_or_dam(csv, child[8])
                foundMatch = 1
                break
            
        if foundMatch == 0 or sire1[2] == '-':
            dataCompleteList[1] = dataCompleteList[3] = dataCompleteList[4] = 1
            x = 7
            while x < 11:
                dataCompleteList[x] = 1
                x += 1

            x = 15
            while x < 23:
                dataCompleteList[x] = 1
                x += 1


    # Locate dam for input animal.
    foundMatch = 0

    if x > 10 and child[10] != '-':
        for m in csv:
            if m[0] == child[10] or m[1] == child[10]:
                dam1 = find_sire_or_dam(csv, child[10])
                foundMatch = 1
                break
            
        if foundMatch == 0 or dam1[2] == '-':
            dataCompleteList[2] = dataCompleteList[5] = dataCompleteList[6] = 1
            
            x = 11
            while x < 15:
                dataCompleteList[x] = 1
                x += 1

            x = 23
            while x < 31:
                dataCompleteList[x] = 1
                x += 1

    # Next generation
    if dataCompleteList[3] == 0:
        foundMatch = 0
        
        x = len(sire1)

        if x > 10:
            for m in csv:
                if m[0] == sire1[8] or m[1] == sire1[8]:
                    # Find sire's sire
                    sire2 = find_sire_or_dam(csv, sire1[8])
                    foundMatch = 1
                    break

        if foundMatch == 0 or sire2[2] == '-':
            dataCompleteList[7] = dataCompleteList[8] = 1
            
            x = 15
            while x < 19:
                dataCompleteList[x] = 1
                x += 1



    if dataCompleteList[4] == 0:

        # Find sire's dam
        foundMatch = 0
        x = len(sire1)

        if x > 10 and sire1[10] != '-':

            for m in csv:
                if m[0] == sire1[10] or m[1] == sire1[10]:
                    dam2 = find_sire_or_dam(csv, sire1[10])
                    foundMatch = 1
                    break

        if foundMatch == 0 or dam2[2] == '-':
            dataCompleteList[9] = dataCompleteList[10] = 1
            
            x = 19
            while x < 22:
                dataCompleteList[x] = 1
                x += 1


    
    if dataCompleteList[5] == 0:
        # find dam's sire
        foundMatch = 0
        x = len(dam1)

        if x > 10 and dam1[8] != '-':
            for m in csv:
                if m[0] == dam1[8] or m[1] == dam1[8]:
                    sire3 = find_sire_or_dam(csv, dam1[8])
                    foundMatch = 1
                    break
                
        if foundMatch == 0 or sire3[2] == '-':
            dataCompleteList[11] = dataCompleteList[12] = 1
            
            x = 23
            while x < 27:
                dataCompleteList[x] = 1
                x += 1


    if dataCompleteList[6] == 0:
        # find dam's dam
        foundMatch = 0
        x = len(dam1)

        if x > 10 and dam1[10] != '-':
            for m in csv:
                if m[0] == dam1[8] or m[1] == dam1[8]:
                    dam3 = find_sire_or_dam(csv, dam1[10])
                    foundMatch = 1
                    break

        if foundMatch == 0 or dam3[2] == '-':
            dataCompleteList[13] = dataCompleteList[14] = 1
            
            x = 27
            while x < 31:
                dataCompleteList[x] = 1
                x += 1


    # Next generation
    if dataCompleteList[7] == 0:
        foundMatch = 0
        x = len(sire2)

        if x > 10 and sire2[8] != '-':
            for m in csv:
                if m[0] == sire2[8] or m[1] == sire2[8]:
                    sire4 = find_sire_or_dam(csv, sire2[8])
                    foundMatch = 1
                    break
                    
        if foundMatch == 0 or sire4[2] == '-':
            dataCompleteList[15] = dataCompleteList[16] = 1

    if dataCompleteList[8] == 0:
        foundMatch = 0

        if x>10 and sire2[10] != '-':
            for m in csv:
                if m[0] == sire2[8] or m[1] == sire2[10]:
                    dam4 = find_sire_or_dam(csv, sire2[10])
                    foundMatch = 1
                    break
                    
        if foundMatch == 0 or dam4[2] == '-':
            dataCompleteList[17] = dataCompleteList[18] = 1


    if dataCompleteList[9] == 0:
        foundMatch = 0

        x = len(dam2)

        if x > 10 and dam2[8] != '-':

            for m in csv:
                if m[0] == dam2[8] or m[1] == dam2[8]:
                    sire5 = find_sire_or_dam(csv, dam2[8])
                    foundMatch = 1
                    break
                
            if foundMatch == 0 or sire5[2] == '-':
                dataCompleteList[19] = dataCompleteList[20] = 1
            
                    
    if dataCompleteList[10] == 0:
        foundMatch = 0
        x = len(dam2)

        if x > 10 and dam2[10] != '-':
            for m in csv:
                if m[0] == dam2[8] or m[1] == dam2[8]:
                    dam5 = find_sire_or_dam(csv, dam2[10])
                    foundMatch = 1
                    break
                        
            if foundMatch == 0 or dam5[2] == '-':
                dataCompleteList[21] = dataCompleteList[22] = 1
            

    if dataCompleteList[11] == 0:
        foundMatch = 0
        x = len(sire3)

        if x > 10 and sire3[8] != '-':
            for m in csv:
                if m[0] == sire3[8] or m[1] == sire3[8]:
                    sire6 = find_sire_or_dam(csv, sire3[8])
                    foundMatch = 1
                    break

            if foundMatch == 0 or sire6[2] == '-':
                dataCompleteList[23] = dataCompleteList[24] = 1
            
            

    if dataCompleteList[12] == 0:
        foundMatch = 0
        x = len(sire3)

        if x > 10 and sire3[10] != '-':
            for m in csv:
                if m[0] == sire3[10] or m[1] == sire3[10]:
                    dam6 = find_sire_or_dam(csv, sire3[10])
                    foundMatch = 1
                    break
                    
            if foundMatch == 0 or dam6[2] == '-':
                dataCompleteList[25] = dataCompleteList[26] = 1
            


    if dataCompleteList[13] == 0:
        foundMatch = 0
        x = len(dam3)

        if x > 10 and dam3[8] != '-':
            for m in csv:
                if m[0] == dam3[8] or m[1] == dam3[8]:
                    sire7 = find_sire_or_dam(csv, dam3[8])
                    foundMatch = 1
                    break
                
        if foundMatch == 0 or sire7[2] == '-':
            dataCompleteList[27] = dataCompleteList[28] = 1
            
    if dataCompleteList[14] == 0:
        foundMatch = 0
        x = len(dam3)

        if x > 10 and dam3[10] != '-':
            for m in csv:
                if m[0] == dam3[10] or m[1] == dam3[10]:
                    dam7 = find_sire_or_dam(csv, dam3[10])
                    foundMatch = 1
                    break
                    
        if foundMatch == 0 or dam7[2] == '-':
            dataCompleteList[29] = dataCompleteList[30] = 1


    # Last generation we care about here
    if dataCompleteList[15] == 0:
        x = len(sire4)

        if x > 10 and sire4[8] != '-':
            for m in csv:
                if m[0] == sire4[8] or m[1] == sire4[8]:
                    sire8 = find_sire_or_dam(csv, sire4[8])
                    break

                
    if dataCompleteList[16] == 0:
        x = len(sire4)

        if x > 10 and sire4[10] != '-':
            for m in csv:
                if m[0] == sire4[10] or m[1] == sire4[10]:
                    dam8 = find_sire_or_dam(csv, sire4[10])
                    break
        
    if dataCompleteList[17] == 0:
        x = len(dam4)

        if x > 10 and dam4[8] != '-':
            for m in csv:
                if m[0] == dam4[8] or m[1] == dam4[8]:
                    sire9 = find_sire_or_dam(csv, dam4[8])
                    break

    if dataCompleteList[18] == 0:
        x = len(dam4)

        if x > 10 and dam4[10] != '-':
            for m in csv:
                if m[0] == dam4[10] or m[1] == dam4[10]:
                    dam9 = find_sire_or_dam(csv, dam4[10])
                    break

                            
    if dataCompleteList[19] == 0:
        x = len(sire5)

        if x > 10 and sire5[8] != '-':
            for m in csv:
                if m[0] == sire5[8] or m[1] == sire5[8]:
                    sire10 = find_sire_or_dam(csv, sire5[8])
                    break
                        
    if dataCompleteList[20] == 0:
        x = len(sire5)

        if x > 10 and sire5[10] != '-':
            for m in csv:
                if m[0] == sire5[10] or m[1] == sire5[10]:
                    dam10 = find_sire_or_dam(csv, sire5[10])
                    break

    if dataCompleteList[21] == 0:
        x = len(dam5)

        if x > 10 and dam5[8] != '-':
            for m in csv:
                if m[0] == dam5[8] or m[1] == dam5[8]:
                    sire11 = find_sire_or_dam(csv, dam5[8])
                    break

    if dataCompleteList[22] == 0:
        x = len(dam5)

        if x > 10 and dam5[10] != '-':
            for m in csv:
                if m[0] == dam5[10] or m[1] == dam5[10]:
                    dam11 = find_sire_or_dam(csv, dam5[10])
                    break

    if dataCompleteList[23] == 0:
        x = len(sire6)

        if x > 10 and sire6[8] != '-':
            for m in csv:
                if m[0] == sire6[8] or m[1] == sire6[8]:
                    sire12 = find_sire_or_dam(csv, sire6[8])
                    break

    if dataCompleteList[24] == 0:
        x = len(sire6)

        if x > 10 and sire6[10] != '-':
            for m in csv:
                if m[0] == sire6[10] or m[1] == sire6[10]:
                    dam12 = find_sire_or_dam(csv, sire6[10])
                    break
                    

    if dataCompleteList[25] == 0:
        x = len(dam6)

        if x > 10 and dam6[8] != '-':
            for m in csv:
                if m[0] == dam6[8] or m[1] == dam6[8]:
                    sire13 = find_sire_or_dam(csv, dam6[8])
                    break
                
    if dataCompleteList[26] == 0:
        x = len(dam6)

        if x > 10 and dam6[10] != '-':
            for m in csv:
                if m[0] == dam6[10] or m[1] == dam6[10]:
                    dam13 = find_sire_or_dam(csv, dam6[10])
                    break
                    
    if dataCompleteList[27] == 0:
        x = len(sire7)

        if x > 10 and sire7[8] != '-':
            for m in csv:
                if m[0] == sire7[8] or m[1] == sire7[8]:
                    sire14 = find_sire_or_dam(csv, sire7[8])
                    break

    if dataCompleteList[28] == 0:
        x = len(sire7)

        if x > 10 and sire7[10] != '-':
            for m in csv:
                if m[0] == sire7[10] or m[1] == sire7[10]:
                    dam14 = find_sire_or_dam(csv, sire7[10])
                    break
                    

    if dataCompleteList[29] == 0:
        x = len(dam7)

        if x > 10 and dam7[8] != '-':
            for m in csv:
                if m[0] == dam7[8] or m[1] == dam7[8]:
                    sire15 = find_sire_or_dam(csv, dam7[8])
                    break

    if dataCompleteList[30] == 0:
        x = len(dam7)

        if x > 10 and dam7[10] != '-':
            for m in csv:
                if m[0] == dam7[10] or m[1] == dam7[10]:
                    dam15 = find_sire_or_dam(csv, dam7[10])
                    break


    # Build rows to be displayed
    lines[0] = ['','']    # spacer

    build_animal_entry(child, 0)
    lines[30] = primary_row
    lines[31] = secondary_row
    lines[32] = tertiary_row

    build_animal_entry(sire1, Sire)
    lines[15] = primary_row
    lines[16] = secondary_row
    lines[17] = tertiary_row

    build_animal_entry(dam1, Dam)
    lines[46] = primary_row
    lines[47] = secondary_row
    lines[48] = tertiary_row

    # Pad the entries with the appropriate number of leading
    # spaces to position them in the correct columns in the
    # spreadsheet.
    for x in range(5):
        lines[15].insert(0, '')
        lines[16].insert(0, '')
        lines[17].insert(0, '')
        lines[46].insert(0, '')
        lines[47].insert(0, '')
        lines[48].insert(0, '')

    # Next Generation
    build_animal_entry(sire2, Sire)
    lines[7] = primary_row
    lines[8] = secondary_row
    lines[9] = tertiary_row

    build_animal_entry(dam2, Dam)
    lines[23] = primary_row
    lines[24] = secondary_row
    lines[25] = tertiary_row

    build_animal_entry(sire3, Sire)
    lines[38] = primary_row
    lines[39] = secondary_row
    lines[40] = tertiary_row

    build_animal_entry(dam3, Dam)
    lines[54] = primary_row
    lines[55] = secondary_row
    lines[56] = tertiary_row

    # Pad the entries with the appropriate number of leading
    # spaces to position them in the correct columns in the
    # spreadsheet.
    for x in range(10):
        lines[7].insert(0, '')
        lines[8].insert(0, '')
        lines[9].insert(0, '')
        lines[23].insert(0, '')
        lines[24].insert(0, '')
        lines[25].insert(0, '')
        lines[38].insert(0, '')
        lines[39].insert(0, '')
        lines[40].insert(0, '')
        lines[54].insert(0, '')
        lines[55].insert(0, '')
        lines[56].insert(0, '')

    lines[7].append('')
    lines[8].append('')
    lines[9].append('')
    lines[23].append('')
    lines[24].append('')
    lines[25].append('')
    lines[38].append('')
    lines[39].append('')
    lines[40].append('')
    lines[54].append('')
    lines[55].append('')
    lines[56].append('')

    # Next Generation
    build_animal_entry(sire4, Sire)
    lines[3] = primary_row
    lines[4] = secondary_row
    lines[5] = tertiary_row

    build_animal_entry(dam4, Dam)
    lines[11] = primary_row
    lines[12] = secondary_row
    lines[13] = tertiary_row

    build_animal_entry(sire5, Sire)
    lines[19] = primary_row
    lines[20] = secondary_row
    lines[21] = tertiary_row

    build_animal_entry(dam5, Dam)
    lines[27] = primary_row
    lines[28] = secondary_row
    lines[29] = tertiary_row

    build_animal_entry(sire6, Sire)
    lines[34] = primary_row
    lines[35] = secondary_row
    lines[36] = tertiary_row

    build_animal_entry(dam6, Dam)
    lines[42] = primary_row
    lines[43] = secondary_row
    lines[44] = tertiary_row

    build_animal_entry(sire7, Sire)
    lines[50] = primary_row
    lines[51] = secondary_row
    lines[52] = tertiary_row

    build_animal_entry(dam7, Dam)
    lines[58] = primary_row
    lines[59] = secondary_row
    lines[60] = tertiary_row

    # Pad the entries with the appropriate number of leading
    # spaces to position them in the correct columns in the
    # spreadsheet.
    for x in range(15):
        lines[3].insert(0, '')
        lines[4].insert(0, '')
        lines[5].insert(0, '')
        lines[11].insert(0, '')
        lines[12].insert(0, '')
        lines[13].insert(0, '')
        lines[19].insert(0, '')
        lines[20].insert(0, '')
        lines[21].insert(0, '')
        lines[27].insert(0, '')
        lines[28].insert(0, '')
        lines[29].insert(0, '')
        lines[34].insert(0, '')
        lines[35].insert(0, '')
        lines[36].insert(0, '')
        lines[42].insert(0, '')
        lines[43].insert(0, '')
        lines[44].insert(0, '')
        lines[50].insert(0, '')
        lines[51].insert(0, '')
        lines[52].insert(0, '')
        lines[58].insert(0, '')
        lines[59].insert(0, '')
        lines[60].insert(0, '')

    for x in range(18):
        lines[30].append('')
        lines[32].append('')

    for x in range(16):
        lines[31].append('')

    for x in range(13):
        lines[10].append('')
        lines[15].append('')
        lines[17].append('')
        lines[46].append('')
        lines[48].append('')

    for x in range(7):
        lines[7].append('')
        lines[9].append('')
        lines[23].append('')
        lines[25].append('')
        lines[38].append('')
        lines[40].append('')
        lines[54].append('')
        lines[56].append('')
            
    for x in range(4):
        lines[22].append('')

    for x in range(3):
        lines[3].append('')
        lines[5].append('')
        lines[11].append('')
        lines[13].append('')
        lines[19].append('')
        lines[21].append('')
        lines[27].append('')
        lines[29].append('')
        lines[34].append('')
        lines[36].append('')
        lines[42].append('')
        lines[44].append('')
        lines[50].append('')
        lines[52].append('')
        lines[58].append('')
        lines[60].append('')

    # Next Generation
    build_animal_entry(sire8, Sire)
    lines[1] = primary_row
    lines[2] = secondary_row
    lines[3].extend(tertiary_row)

    build_animal_entry(dam8, Dam)
    lines[5].extend(primary_row)
    lines[6] = secondary_row
    lines[7].extend(tertiary_row)

    build_animal_entry(sire9, Sire)
    lines[9].extend(primary_row)
    lines[10] = secondary_row
    lines[11].extend(tertiary_row)

    build_animal_entry(dam9, Dam)
    lines[13].extend(primary_row)
    lines[14] = secondary_row
    lines[15].extend(tertiary_row)
    
    build_animal_entry(sire10, Sire)
    lines[17].extend(primary_row)
    lines[18] = secondary_row
    lines[19].extend(tertiary_row)

    build_animal_entry(dam10, Dam)
    lines[21].extend(primary_row)
    lines[22] = secondary_row
    lines[23].extend(tertiary_row)

    build_animal_entry(sire11, Sire)
    lines[25].extend(primary_row)
    lines[26] = secondary_row
    lines[27].extend(tertiary_row)

    build_animal_entry(dam11, Dam)
    lines[29].extend(primary_row)
    lines[30].extend(secondary_row)
    lines[31].extend(tertiary_row) 

    build_animal_entry(sire12, Sire)
    lines[32].extend(primary_row)
    lines[33] = secondary_row
    lines[34].extend(tertiary_row)

    build_animal_entry(dam12, Dam)
    lines[36].extend(primary_row)
    lines[37].extend(secondary_row)
    lines[38].extend(tertiary_row) 

    build_animal_entry(sire13, Sire)
    lines[40].extend(primary_row)
    lines[41] = secondary_row
    lines[42].extend(tertiary_row)
 
    build_animal_entry(dam13, Dam)
    lines[44].extend(primary_row)
    lines[45] = secondary_row
    lines[46].extend(tertiary_row) 

    build_animal_entry(sire14, Sire)
    lines[48].extend(primary_row)
    lines[49] = secondary_row
    lines[50].extend(tertiary_row)

    build_animal_entry(dam14, Dam)
    lines[52].extend(primary_row)
    lines[53] = secondary_row
    lines[54].extend(tertiary_row) 

    build_animal_entry(sire15, Sire)
    lines[56].extend(primary_row)
    lines[57] = secondary_row
    lines[58].extend(tertiary_row)

    build_animal_entry(dam15, Dam)
    lines[60].extend(primary_row)
    lines[61] = secondary_row
    lines[62] = tertiary_row

    # Pad the entries with the appropriate number of leading
    # spaces to position them in the correct columns in the
    # spreadsheet.
    for x in range(20):
        lines[1].insert(0, '')
        lines[2].insert(0, '')
        lines[6].insert(0, '')
        lines[10].insert(0, '')
        lines[14].insert(0, '')
        lines[18].insert(0, '')
        lines[22].insert(0, '')
        lines[26].insert(0, '')
        lines[33].insert(0, '')
        lines[41].insert(0, '')
        lines[45].insert(0, '')
        lines[49].insert(0, '')
        lines[53].insert(0, '')
        lines[57].insert(0, '')
        lines[61].insert(0, '')
        lines[62].insert(0, '')

    for x in range(19):
        lines[37].insert(0, '')

    # Add DOB and tattoo information for requested animal
    lines[51].pop(2)
    lines[51].insert(2, get_animal_dob(child))

    lines[53].pop(2)
    lines[53].insert(2, get_animal_r_tattoo(child))

    lines[55].pop(2)
    lines[55].insert(2, get_animal_l_tattoo(child))

    lines[57].pop(2)
    lines[57].insert(2, get_animal_sex(child))

    # Build CSV file
    # Create an output file for the pedigree.
    if noArgs == 0:
        print "Creating .csv file"

    fname = nameIn.replace(" ","_") + ".csv"
    
    f = open(fname, 'w')
    
    for x in range(63):
        row = ['"%s"' % i for i in lines[x]]
        f.write(",".join(row))
        f.write("\n")

    f.close

    if noArgs == 0:
        print fname + " complete. Import into your spreadsheet."
    else:
        tkMessageBox.showinfo("Done", fname + " complete. Import into your spreadsheet.")
   
    sys.exit(0)
