#! /usr/bin/env python
#
# -*- coding: latin1 -*-
"""
getanimal.py

 A script for retrieving extended pedigree information
 from the AHCA web site.

 USAGE:

     getanimal.py [csv file name] <"Animal Name">
     
                        or

     getanimal.py [csv file name] <Registration Number>
     
Copyright (C) 2007 - 2021 Ben E. Schmidtke Jr.

Revised 10/34/21 Add checks for registrations with NZ, AU, SCxx, SCxxx
                 Fix tattoo parsing for case of left and right tattoos on same line

Revised 03/29/16 Fix gzip encoding. AHCA apparently just started using it.

Revised 08/18/13 Correct problem entering Scottish or Canadian 
                 registration numbers
                 
Revised 03/09/13 Correct problems with names that don't have links
                 Correct problems with strings like (AI, D) in names

Revised 01/01/09  Add windowed environment support

Revised 12/26/08  Revised to work with new AHCA web site pages.

Revised 12/31/07  Added Date Of birth, left and Right tattoos.

Revised 07/18/07  Revised to fix an index error in AHCAPageParser and
                  added comments.

Revised 05/20/07  Revised to add sire and dam registration numbers
                  so variations in names don't screw up the pedigree
                  generation

Revised 04/26/07 to automatically update the database file.

"""
import urllib2, sgmllib, re
import tkMessageBox
import gzip
from StringIO import StringIO
from Tkinter  import *

"""
 Simple dialog support
"""

class Dialog(Toplevel):
    """
    Class to open dialog
    """
    def __init__(self, parent, title = None):

        """
        Initialize a dialog.

        Arguments:

            parent -- a parent window (the application window)

            title -- the dialog title
        """
        
        Toplevel.__init__(self, parent)
        self.transient(parent)

        if title:
            self.title(title)

        self.parent = parent

        self.result = None
        self.fastSearch = 0

        body = Frame(self)
        self.initial_focus = self.body(body)
        body.pack(padx=5, pady=5)

        self.buttonbox()

        self.wait_visibility() # window needs to be visible for the grab
        self.grab_set()

        if not self.initial_focus:
            self.initial_focus = self

        self.protocol("WM_DELETE_WINDOW", self.cancel)

        if self.parent is not None:
            self.geometry("+%d+%d" % (parent.winfo_rootx()+50,
                                      parent.winfo_rooty()+50))

        self.initial_focus.focus_set()

        self.wait_window(self)

    def destroy(self):
        """
        Destroy the window
        """
        self.initial_focus = None
        Toplevel.destroy(self)

    #
    # construction hooks

    def body(self, master):
        """
        create dialog body.

        return widget that should have initial focus.
        This method should be overridden, and is called
        by the __init__ method.
        """
        pass


    def buttonbox(self):
        """
        add standard button box.

        override if you do not want the standard buttons
        """

        box = Frame(self)

        w = Button(box, text="OK", width=10, command=self.ok, default=ACTIVE)
        w.pack(side=LEFT, padx=5, pady=15)
        w = Button(box, text="Cancel", width=10, command=self.cancel)
        w.pack(side=LEFT, padx=5, pady=15)

        self.bind("<Return>", self.ok)
        self.bind("<Escape>", self.cancel)

        self.cb = Checkbutton(box, text="Fast Search", command=self.fastSearchToggle)
        self.cb.pack(side=LEFT, padx=5, pady=5)
        self.cb.deselect()

        box.pack()


    #
    # standard button semantics

    # Checkbox doesn't appear to automagically toggle variable
    def fastSearchToggle(self, event=None):
        if self.fastSearch==0:
            self.fastSearch = 1
        else:
            self.fastSearch = 0

    def ok(self, event=None):

        if not self.validate():
            self.initial_focus.focus_set() # put focus back
            return

        self.withdraw()
        self.update_idletasks()
        self.apply()
        self.cancel()

    def cancel(self, event=None):

        # put focus back to the parent window
        if self.parent is not None:
            self.parent.focus_set()
        self.destroy()

    #
    # command hooks

    def validate(self):
        """
        validate the data

        This method is called automatically to validate the data before the
        dialog is destroyed. By default, it always validates OK.
        """

        return 1 # override

    def apply(self):
        """
        process the data

        This method is called automatically to process the data, *after*
        the dialog is destroyed. By default, it does nothing.
        """

        pass # override

    def isFastSearch(self):
        return self.fastSearch

# --------------------------------------------------------------------
# convenience dialogues

class _QueryDialog(Dialog):

    def __init__(self, title, prompt,
                 initialvalue=None,
                 parent = None):

        if not parent:
            import Tkinter
            parent = Tkinter._default_root

        self.prompt   = prompt

        self.initialvalue = initialvalue

        Dialog.__init__(self, parent, title)

    def destroy(self):
        self.entry = None
        Dialog.destroy(self)

    def body(self, master):

        w = Label(master, text=self.prompt, justify=LEFT)
        w.grid(row=0, columnspan=32, padx=5, sticky=W)

        self.entry = Entry(master, name="entry")
        self.entry.grid(row=1, columnspan=32, padx=5, sticky=W+E)

        if self.initialvalue:
            self.entry.insert(0, self.initialvalue)
            self.entry.select_range(0, END)

        return self.entry

    def validate(self):

        import tkMessageBox

        try:
            result = self.getresult()
        except ValueError:
            tkMessageBox.showwarning(
                "Illegal value",
                self.errormessage + "\nPlease try again",
                parent = self
            )
            return 0

        self.result = result
        self.fastSearch = Dialog.isFastSearch(self)

        return 1


class _QueryString(_QueryDialog):
    def __init__(self, *args, **kw):
        if kw.has_key("show"):
            self.__show = kw["show"]
            del kw["show"]
        else:
            self.__show = None
        _QueryDialog.__init__(self, *args, **kw)

    def body(self, master):
        entry = _QueryDialog.body(self, master)
        self.fastSearch = _QueryDialog.isFastSearch(self)

        if self.__show is not None:
            entry.configure(show=self.__show)

        return entry

    def getresult(self):
        return self.entry.get()

    def isFastSearch(self):
        return self.fastSearch

class displayProgress(object):
    def __init__(self):
        "Initialize this class"
        import Tkinter
        
        self.root = Tk()
        self.sb   = Tkinter.Scrollbar()
        self.lb   = Tkinter.Listbox(width = 60)

        self.sb.pack(side=Tkinter.RIGHT, fill=Tkinter.Y)
        self.lb.pack(side=Tkinter.LEFT, fill=Tkinter.Y)

        self.sb.config(command=self.lb.yview)
        self.lb.config(yscrollcommand=self.sb.set)
        self.root.update()
        
    def addLine(self, listItem):
        "Add a line to the listbox"
        
        self.lb.insert(END, listItem)
        self.root.update()

    def destroy(self):
        self.entry = None
        object.destroy(self)

class displayDbUpdate(object):
    def __init__(self):
        "Initialize this class"
        import Tkinter
        
        self.root = Tk()
        self.sb   = Tkinter.Scrollbar()
        self.lb   = Tkinter.Listbox(width = 60)

        self.sb.pack(side=Tkinter.RIGHT, fill=Tkinter.Y)
        self.lb.pack(side=Tkinter.LEFT, fill=Tkinter.Y)

        self.sb.config(command=self.lb.yview)
        self.lb.config(yscrollcommand=self.sb.set)
        self.root.update()
        
    def addLine(self, listItem):
        "Add a line to the listbox"
        
        self.lb.insert(END, listItem)
        self.root.update()

    def destroy(self):
        self.entry = None
        object.destroy(self)


#------------------------------------------------------------------------

class AhcaParser(sgmllib.SGMLParser):
    "A simple parser class."

    def parse(self, s):
        "Parse the given string 's'."
        self.feed(s)
        self.close()

    def __init__(self, verbose=0):
        "Initialise an object, passing 'verbose' to the superclass."

        sgmllib.SGMLParser.__init__(self, verbose)
        self.hyperlinks = []
        self.descriptions = []
        self.text = []
        self.inside_a_element = 0
        self.starting_description = 0

    def start_a(self, attributes):
        "Process a hyperlink and its 'attributes'."

        for name, value in attributes:
            if name == "href":
                self.hyperlinks.append(value)
                self.inside_a_element = 1
                self.starting_description = 1

    def end_a(self):
        "Record the end of a hyperlink."

        self.inside_a_element = 0

    def handle_data(self, data):
        "Handle the textual 'data'."
        if self.inside_a_element:
            if self.starting_description:
                self.descriptions.append(data)
                self.starting_description = 0
            else:
                self.descriptions[-1] += data
                

    def get_hyperlinks(self):
        "Return the list of hyperlinks."

        return self.hyperlinks

    def get_descriptions(self):
        "Return a list of descriptions."

        return self.descriptions


"""
  class initialPageParser
  
  A class whose only function is to handle the first page
  in the string of requests.
  
"""
class initialPageParser(object):
    
    def parsePage(self, dpw, s, param):

        self.search_link = []
        self.lines       = []
        fLen = 0
        haveLink = 0
        
        if s.find("Service Unavailable", 0, 27) >= 0:
            if dpw == None:
                print "Service Unavailable"
            else:
                tkMessageBox.showerror("Service Unavailable")
            
        strings = StringIO(s)

        numLines = len(strings.readlines())
        strings.seek(0)
        self.lines = strings.readlines()

        for x in range(numLines):
            if re.search("i4.dll", self.lines[x]):
                if re.search("1=232B2F&2=2420&3=56&5=2B3C2B3C3A&6", self.lines[x]):

                    temp = self.lines[x].split('"')
                    temp2 = temp[2].split('>')
                    animalName = temp2[1].replace("</a", "")

                    if dpw == None:
                        print animalName
                    else:
                        displayProgress.addLine(dpw, animalName)

                    self.search_link = temp[1]

#                    print self.search_link
                    
                    break

    def get_link(self):
        return self.search_link


"""
  class AHCAPageParser
  
  The class that extracts the necessary pedigree information
  from a animal's page.
  
"""
class AHCAPageParser(object):

    def __init__(self):
        self.have_sire  = 0
        

    def parsePage(self, s):

        self.lines      = []
        self.name       = []
        self.reg_number = []
        self.color      = []
        self.sex        = []
        self.temp_sex   = []
        self.sire_name  = []
        self.dam_name   = []
        self.sire_link  = []
        self.dam_link   = []
        self.dob        = []
        self.l_tattoo   = []
        self.r_tattoo   = []
        
        strings = StringIO(s)

        numLines = len(strings.readlines())
        strings.seek(0)

        for i in range(numLines):
            self.lines.append(strings.readline())
    
        # Try and process the page.
        # The class should have been defined first, remember.

        myparser = AhcaParser()
        myparser.parse(s)

        # Get the hyperlinks.
        self.descriptions = myparser.get_descriptions()
        self.links        = myparser.get_hyperlinks()
        numLinks          = self.links.__len__()
        no_sire           = 0
        no_dam            = 0
        temp_sire_link    = []
        temp_dam_link     = []
        linkIndex         = 0

        # Find the line that contains the Sire link
        hasPedTree = 0

        for x in range(numLines):
            if re.search("PedTree", self.lines[x]):
                temp_strings = self.lines[x].split('"')
                hasPedTree = 1

        temp_dam_link = "no_link"
        temp_sire_link = "no_link"

        if hasPedTree == 1:

            # We now have the line that contains the links
            # Identify the Sire and Dam links. Since this
            # information was lost when we extracted the links
            # and descriptions, we have to do this the hard way!
            for x in range(2500):
                if re.search("Sire: ", temp_strings[x]):
                    try:
                        temp_sire_link = temp_strings[x + 1]
                        temp_sire_name = temp_strings[x + 2]
                    except IndexError:
                        temp_sire_link = "no_link"

                elif re.search("Dam: ", temp_strings[x]):
                    try:
                        temp_dam_link = temp_strings[x + 1]
                        temp_dam_name = temp_strings[x + 2]
                    except IndexError:
                         temp_dam_link = "no_link"

                    # Once we have the dam, we can quit looking
                    break

            # We want the Sire and Dam links
            for x in range(numLinks):
                if temp_sire_link == self.links[x]:
                    try:
#                    self.sire_name = self.descriptions[x].replace("AI,D","AI D")
#                    self.sire_name = self.descriptions[x].replace("ET,D","ET D")
#                    self.sire_name = self.descriptions[x].replace("t,D","t D")
                        self.sire_name = self.descriptions[x].replace(",", " ")

                    except IndexError:
                        self.sire_name = ''

                elif temp_dam_link == self.links[x]:
                    try:
#                    self.dam_name = self.descriptions[x].replace("AI,D","AI D")
#                    self.dam_name = self.descriptions[x].replace("ET,D","ET D")
#                    self.dam_name = self.descriptions[x].replace("t,D","t D")
                        self.dam_name = self.descriptions[x].replace(","," ")   

                    except IndexError:
                        self.dam_name = ''

        # Extract the relevant data from the page.
                
        for x in range(numLines):
            try:
                if re.search("<title>Highland Cattle Animal Details ", self.lines[x]):
            #                    self.name = self.lines[x].replace("  <title>Highland Cattle Animal Details ", "").replace("*", "").replace("AI,D","AI D").replace("ET,D","ET D").replace("t,D","t D").replace("</title>\r\n", "")
                    self.name = self.lines[x].replace("  <title>Highland Cattle Animal Details ", "").replace(",", " ").replace("</title>\r\n", "")

                if re.search("Regn. No.:", self.lines[x]):
                    self.reg_number = self.lines[x + 1].replace("&nbsp;\r\n", "")

                if re.search("Sex:", self.lines[x]):
                    self.temp_sex = self.lines[x + 1].replace("&nbsp;\r\n", "")
                    
                if re.search("Color:", self.lines[x]):
                    self.color = self.lines[x + 1].replace("&nbsp;\r\n", "")

                # Added DOB, LE Tattoo, RE Tattoo 12/31/07
                if re.search("Birth Date:", self.lines[x]):
                    self.dob = self.lines[x + 1].replace("&nbsp;\r\n", "")
                    
                if re.search("Tattoo:", self.lines[x]):
                    if re.search("Right", self.lines[x + 1]):
                        if re.search("Left", self.lines[x + 1]):
                            l = self.lines[x + 1].replace("(", ",").replace(") ", ",").split(',')
                        
                            self.r_tattoo = l[0]
                            self.l_tattoo = l[2]
                                
                    elif re.search("Left", self.lines[x + 1]):
                        l = self.lines[x + 1].replace("(", ",").replace(") ", ",").split(',')
                        
                        self.l_tattoo = l[0]
                    elif re.search("Right", self.lines[x + 1]):
                        l = self.lines[x + 1].replace("(", ",").replace(") ", ",").split(',')
                        
                        self.r_tattoo = l[0]

            except IndexError:
                break


            if no_sire == 0:
                self.sire_link = temp_sire_link

            if no_dam == 0:
                self.dam_link = temp_dam_link
            
        try:
            if self.temp_sex == "Female":
                self.sex = "Cow"
            elif self.temp_sex == "Male":
                self.sex = "Bull"
            else:
                self.sex = self.temp_sex

        except TypeError:
            self.sex = ""



    def get_sire_link(self):
        return self.sire_link

    def get_dam_link(self):
        return self.dam_link

    def get_sire_name(self):
        try:
            return self.sire_name
        except AttributeError:
            return ''
        
    def get_dam_name(self):
        try:
            return self.dam_name
        except AttributeError:
            return ''

    def get_color(self):
        try:
            color = self.color
        except IndexError:
            color = '-'
        return color

    def get_registration(self):
        try:
            number = self.reg_number
        except IndexError:
            number = '-'
        return number

    def get_sex(self):
        return self.sex

    def get_name(self):
        try:
            name = self.name
        except IndexError:
            name = ""
        return name

    def get_pageError(self):
        return self.pageError

    # New fields
    def get_dob(self):
        try:
            dob = self.dob
        except IndexError:
            dob = '-'
        return dob

    def get_left_tattoo(self):
        try:
            left_tattoo = self.l_tattoo
        except IndexError:
            left_tattoo = '-'
        return left_tattoo

    def get_right_tattoo(self):
        try:
            right_tattoo = self.r_tattoo
        except IndexError:
            right_tattoo = '-'
        return right_tattoo


import sys

# Check to see if the key already is in the dictionary. Return True/False status.
def key_exists(dictionary, key):

    master_keys = dictionary.keys()
    haveKey = 0
    
    for x in range(dictionary.__len__()):
        try:
            if key == master_keys[x]:
                haveKey = 1
                break
            
        except IndexError:
            break

    return haveKey




# Build an animal's entry (1 row in the file)
def build_row(dpw, url, dictionary, addEntry, updateSireKey, updateDamKey, keyList, dummyKey):
    
    raw_row = []
    validEntry = 0
        

    if url == "no_link":
        raw_row.append(dummyKey)
        raw_row.append('')
        raw_row.append('')
        raw_row.append('')
        raw_row.append('')
        raw_row.append('')
        raw_row.append('')
        raw_row.append('')
        raw_row.append('')

        # New Fields
        raw_row.append('')
        raw_row.append('')
        raw_row.append('')
    else:
        f = urllib2.urlopen(url)
        s = f.read()
        f.close()

        myAHCAPageParser.parsePage(s)
        
        regNumber = myAHCAPageParser.get_registration()

        try:
            if regNumber.isdigit():
                raw_row.append(regNumber)          # U.S. registration
                raw_row.append('')                 # Foreign registration
            else:
                raw_row.append('')                 # U.S. registration
                raw_row.append(regNumber)          # Foreign registration
        except:
                raw_row.append('')                 # No registration
                raw_row.append('')
                
        try:
            raw_row.append(myAHCAPageParser.get_name().replace(",", " "))

        except AttributeError:
            print myAHCAPageParser.get_name()
            sys.exit(1)

        # New Fields
        raw_row.append(myAHCAPageParser.get_dob())
        raw_row.append(myAHCAPageParser.get_left_tattoo())
        raw_row.append(myAHCAPageParser.get_right_tattoo())
        
        raw_row.append(myAHCAPageParser.get_sex())
        raw_row.append(myAHCAPageParser.get_color())
        raw_row.append('')                     # Sire's registration number
        raw_row.append(myAHCAPageParser.get_sire_name())
        raw_row.append('')                     # Dam's registration number
        raw_row.append(myAHCAPageParser.get_dam_name())

    # Update the dictionary
    if raw_row[0] != '':
        key = raw_row[0]
        validEntry = 1
    elif raw_row[1] != '':
        key = raw_row[1]
        validEntry = 1

    # Print the name out so the user knows that we're still alive.
    if dpw == None:
        print key, raw_row[2]
    else:
        try:
            displayProgress.addLine(dpw, key + " " + raw_row[2])
        except TypeError:
            displayProgress.addLine(dpw, "missing data")

    if addEntry == 1:
        if validEntry == 1:
            dictionary[key] = raw_row
            keyList.append(key)
            
    if validEntry == 1:

        # Fill in the registration numbers for the sire and dam.
        if url != "no_link":
            if updateSireKey != 0:
                dictionary[updateSireKey][8] = key
            elif updateDamKey != 0:
                dictionary[updateDamKey][10] = key
        else:
            if updateSireKey != 0:
                dictionary[updateSireKey][8] = ''
                dictionary[updateSireKey][9] = ''
            elif updateDamKey != 0:
                dictionary[updateDamKey][10] = ''
                dictionary[updateDamKey][11] = ''
                    
    return key

    
def askstring(title, prompt, **kw):
    global fastSearch

    """
    get a string from the user

    Arguments:

        title -- the dialog title
        prompt -- the label text
        **kw -- see SimpleDialog class

    Return value is a string
    """
    d = _QueryString(title, prompt, **kw)

    fastSearch = d.isFastSearch()

    return d.result

def getAnimalInfo():

    root = Tk()
    root.update()

    inputStr = askstring("Get Animal", "Enter the Name or Registration #")

    root.destroy()

    return inputStr


"""
  The entry point.........
"""
if __name__=="__main__":
    global fastSearch

    regNum = FALSE
    
    # Define the master database file name. (default)
    dataBaseFile = "master_db.csv"

    # Setting this to a non-zero value will eliminate any web queries in a part
    # of the tree once an animal already exists in the database. Up side is
    # the search goes much faster since we don't have to ask for 30+ pages. The
    # down side is that the pedigree tree will have a hole or holes if animals
    # are missing beyond that point. 
    fastSearch = 0

    # Attempt to determine whether windowed or command line environment
    noArgs = 0
    
    if len(sys.argv)==2:
        animalName = sys.argv[1]

    elif len(sys.argv)==3:
        dataBaseFile = sys.argv[1]
        animalName = sys.argv[2]
    else:
        animalName = getAnimalInfo()
        noArgs = 1

    if noArgs == 0:
        dpw = None
    else:
        dpw = displayProgress() # Create the window for further messages
    
    if animalName == None:
        if dpw == None:
            print >> sys.stderr, "Usage: %s [database file] <animal name>" % sys.argv[0]
        else:
            tkMessageBox.showwarning("Error", "Usage: %s [database file] <animal name>" % sys.argv[0])
            
        sys.exit(1)

# Form old style request
#    base_url = "http://www.highlandcattleusa.org/admin/members/herdbook/"

#    if animalName.isdigit():
#        searchUrl = base_url + "resultsanimals.aspx?RegNo=" + str(animalName)
#        print searchUrl
#    else:
#        searchUrl = base_url + "resultsanimals.aspx?Name=" + animalName.replace(" ", "%20")
#        print searchUrl
        
    # Form request data for new style request

    if animalName.isdigit():
        regNum = TRUE
    else:
        tempName = animalName.upper()
        
        if tempName.find("SC", 0, 22) == 0:
            if tempName.lstrip('SC').isdigit():
                regNum = TRUE
        elif tempName.find("CA", 0, 22) == 0:
            if tempName.lstrip('CA').isdigit():
                regNum = TRUE
        elif tempName.find("NZ", 0, 22) == 0:
            if tempName.lstrip('NZ').isdigit():
                regNum = TRUE
        elif tempName.find("AU", 0, 22) == 0:
            if tempName.lstrip('AU').isdigit():
                regNum = TRUE
        elif tempName.find("SCNEA", 0, 22) == 0:
            if tempName.lstrip('SCNEA').isdigit():
                regNum = TRUE
        elif tempName.find("SCNEP", 0, 22) == 0:
            if tempName.lstrip('SCNEP').isdigit():
                regNum = TRUE
        elif tempName.find("SCWLC", 0, 22) == 0:
            if tempName.lstrip('SCWLC').isdigit():
                regNum = TRUE
        elif tempName.find("SCDWW", 0, 22) == 0:
            if tempName.lstrip('SCDWW').isdigit():
                regNum = TRUE
        elif tempName.find("SCMPH", 0, 22) == 0:
            if tempName.lstrip('SCMPH').isdigit():
                regNum = TRUE
        elif tempName.find("SCRMP", 0, 22) == 0:
            if tempName.lstrip('SCRMP').isdigit():
                regNum = TRUE
        elif tempName.find("SCGRC", 0, 22) == 0:
            if tempName.lstrip('SCGRC').isdigit():
                regNum = TRUE

    if regNum:
        animalData = "anim_name=&anim_ident=" + str(animalName)        
    else:
        animalData = "anim_name="+ animalName.replace(" ", "+") + "&anim_ident="
        
    requestData = animalData + "&anim_tattoo=&calving_year=&current_owner_ident=&as_select_1=0&as_select_2=0&as_select_3=0&soc_an_field_1_mult=&as_grp_select_1=10&as_type_radio=10&as_horn_radio=10&as_rego_radio=10&sire_name=&dam_name=&as_sort_by=anim_name&sort_mode=asc"

    requestDataLen = len(requestData)

    # Try new style page

    searchUrl = "http://abri.une.edu.au/online/cgi-bin/i4.dll?1=232B2F&2=2428&3=56&5=2B3C2B3C3A"

    request2 =urllib2.Request(searchUrl)

    # Fill in the request header information
    request2.add_header('Host', 'abri.une.edu.au')
    request2.add_header('Accept', 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8')
    request2.add_header('Accept-Encoding', 'gzip,deflate')
    request2.add_header('Accept-Charset', 'ISO-8859-1,utf-8;q=0.7,*;q=0.7')
    request2.add_header('Keep-Alive', '300')
    request2.add_header('Content-Type', 'application/x-www-form-urlencoded')
    request2.add_header('Content-Length', str(requestDataLen))
    request2.add_data(requestData)

    # We now have a request that should return a page containing the request that we really need
    # to make. Give it a try....
    try:
        response = urllib2.urlopen(request2)
 
        if response.info().get('Content-Encoding') == 'gzip':
            buf = StringIO(response.read())
            f = gzip.GzipFile(fileobj=buf)
            s = f.read()
        else:
            s = response.read()
 
        response.close()
        
    except (IOError, OSError):
        if dpw == None:
            print "New style request - Error reading page. Quitting."
        else:
            tkMessageBox.showwarning("Problem", "Error reading page. Quitting.")
            
        sys.exit(1)

    # Now, we have to find the line containing the animal's name. That line reads something like:
    #
    #<a href="i4.dll?1=232B2F&2=2420&3=56&5=2B3C2B3C3A&6=595D5D5822232623&9=53535C">Animal Name</a>&nbsp;
    #
    # We want everything between quotes.
    # Right now, it appears that the line of interest is the last one that starts:
    #
    #<a href="i4.dll?1=232B2F&2=2420&3=56&5=2B3C2B3C3A&6=
    #
    try:
        # Instantiate the parser class
        basePageParser = initialPageParser()

        # Get the initial page for the animal
        basePageParser.parsePage(dpw, s, animalName)

    except (IOError, OSError):
        if dpw == None:
            print "Error parsing initial page."
        else:
            tkMessageBox.showwarning("Problem", "Error parsing initial page.")
            
        sys.exit(1)

    requestString = basePageParser.get_link()

#    print requestString

    base_url = "http://abri.une.edu.au/online/cgi-bin/"

    try:
        searchUrl = base_url + requestString

    except (TypeError):
        if dpw == None:
            print "Animal not found"
        else:
            tkMessageBox.showwarning("Problem", "Animal not found.")

        sys.exit(1)

    request2 =urllib2.Request(searchUrl)

    # Fill in the request header information
    request2.add_header('Host', 'abri.une.edu.au')
    request2.add_header('Accept', 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8')
    request2.add_header('Accept-Encoding', 'gzip,deflate')
    request2.add_header('Accept-Charset', 'ISO-8859-1,utf-8;q=0.7,*;q=0.7')
    request2.add_header('Keep-Alive', '300')

    try:
        response = urllib2.urlopen(request2)
 
        if response.info().get('Content-Encoding') == 'gzip':
            buf = StringIO(response.read())
            f = gzip.GzipFile(fileobj=buf)
            s = f.read()
        else:
            s = response.read()
 
        response.close()
        
    except (IOError, OSError):
        if dpw == None:
            print "Secondary request - Error reading page. Quitting."
        else:
            tkMessageBox.showwarning("Problem", "Secondary request - Error reading page.")
            
        sys.exit(1)

    dataPresentList = []

    # Attempt to open the master database. Create it if
    # it doesn't exist
    try:
        # Check for the existance of the database file
        dbFile = open(dataBaseFile, 'r')
        dbFile.close()

    except (IOError, OSError):
        # Create the non-existant database file
        dbFile = open(dataBaseFile, 'w')
        print >> dbFile, "Reg #,","Other Reg#,","Name,","DOB,","LE Tattoo,","RE Tattoo,","Sex,","Color,","Sire Reg,","Sire,","Dam Reg,","Dam"
        dbFile.close()

    # Update database file
    # First, figure out how big it is.
    dbFile = open(dataBaseFile, 'r')
            
    fLen = len(dbFile.readlines())
    dbFile.seek(0)

    # Read the file in.
    s = dbFile.read().strip('"')
    
    dbFile.close()

    csv=[]
    raw = []
    dbDict = dict()
    raw_row = ['', '', '', '', '', '', '', '', '', '', '', '']

    # Remove all quotes and newlines and create a list of the lines in the
    # input file
    raw = s.replace('"',"").splitlines(0)

    for x in range(fLen):
        csv.append(raw[x].split(','))
                   
    # Build dictionary of database

    for x in range(fLen):
        l = csv[x]
        validEntry = 0
        
        if l[0] != '':
            key = l[0]
            validEntry = 1
        elif l[1] != '':
            key = l[1]
            validEntry = 1
            
        if validEntry == 1:
            dbDict[key] = l


    masterKeys = dbDict.keys()

    # Create a list of flags to indicate whether we already
    # have the data for an animal in our local database
    for x in range(31):
        dataPresentList.append(0)
    
    localDict = dict() # We'll build a dictionary as we go
    keyList = list()
    
    # In order to keep things simple, we need to at least create an entry
    # in the dictionary we are building so that the row indexes remain
    # consistant. We create an initial dummy key for that here.
    # The assumption we make here is that there are no registration
    # numbers less than 32
    fillKey = 0
    
    myAHCAPageParser = AHCAPageParser()

    url = base_url + basePageParser.get_link()

    newKey = build_row(dpw, url, localDict, 1, 0, 0, keyList, fillKey)
    fillKey = fillKey + 1
    
    if fastSearch > 0:
        if key_exists(dbDict, newKey) == 1:
            for x in range(31):
                dataPresentList[x] = 1
    
    if dataPresentList[1] == 0:
        try:
            sire1_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire1_link.find("%") >= 0:
                sire1_link = "no_link"
        except TypeError:
            sire1_link = "no_link"
        
    if dataPresentList[2] == 0:
        try:
            dam1_link = base_url+myAHCAPageParser.get_dam_link()

            if dam1_link.find("%") >= 0:
                dam1_link = "no_link"
        except TypeError:
            dam1_link = "no_link"
            
    # sire 1
    if dataPresentList[1] == 0:
        newKey = build_row(dpw, sire1_link, localDict, 1, keyList[0], 0, keyList, fillKey)

        if fastSearch > 0:
            if key_exists(dbDict, newKey) == 1:
                dataPresentList[3] = dataPresentList[4] = 1

                x = 7
                while x < 11:
                    dataPresentList[x] = 1
                    x += 1

                x = 15
                while x < 23:
                    dataPresentList[x] = 1
                    x += 1
    
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    

    #sire 2
    if dataPresentList[3] == 0:
        try:
            sire2_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire2_link.find("%") >= 0:
                sire2_link = "no_link"

        except TypeError:
            sire2_link = "no_link"
        
    # dam 2
    if dataPresentList[4] == 0:
        try:
            dam2_link = base_url+myAHCAPageParser.get_dam_link()

            if dam2_link.find("%") >= 0:
                dam2_link = "no_link"

        except TypeError:
            dam2_link = "no_link"
            
    
    # dam 1
    if dataPresentList[2] == 0:
        newKey = build_row(dpw, dam1_link, localDict, 1, 0, keyList[0], keyList, fillKey)

        if fastSearch > 0:
            if key_exists(dbDict, newKey) == 1:
                dataPresentList[5] = dataPresentList[6] = 1

                x = 11
                while x < 15:
                    dataPresentList[x] = 1
                    x += 1

                x = 23
                while x < 31:
                    dataPresentList[x] = 1
                    x += 1
    
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1

    # sire 3
    if dataPresentList[5] == 0:
        try:
            sire3_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire3_link.find("%") >= 0:
                sire3_link = "no_link"

        except TypeError:
            sire3_link = "no_link"
        
    # dam 3
    if dataPresentList[6] == 0:
        try:
            dam3_link = base_url+myAHCAPageParser.get_dam_link()

            if dam3_link.find("%") >= 0:
                dam3_link = "no_link"

        except TypeError:
            dam3_link = "no_link"
            
    
    # sire 2
    if dataPresentList[3] == 0:
        newKey = build_row(dpw, sire2_link, localDict, 1, keyList[1], 0, keyList, fillKey)
    
        if fastSearch > 0:
            if key_exists(dbDict, newKey) == 1:
                dataPresentList[7] = dataPresentList[8] = 1

                x = 15
                while x < 19:
                    dataPresentList[x] = 1
                    x += 1

    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    # sire 4
    if dataPresentList[7] == 0:
        try:
            sire4_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire4_link.find("%") >= 0:
                sire4_link = "no_link"

        except TypeError:
            sire4_link = "no_link"
        
    # dam 4
    if dataPresentList[8] == 0:
        try:
            dam4_link = base_url+myAHCAPageParser.get_dam_link()

            if dam4_link.find("%") >= 0:
                dam4_link = "no_link"

        except TypeError:
            dam4_link = "no_link"
            
    
    # dam 2
    if dataPresentList[4] == 0:
        newKey = build_row(dpw, dam2_link, localDict, 1, 0, keyList[1], keyList, fillKey)
    
        if fastSearch > 0:
            if key_exists(dbDict, newKey) == 1:
                dataPresentList[9] = dataPresentList[10] = 1

                x = 19
                while x < 23:
                    dataPresentList[x] = 1
                    x += 1

    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    # sire 5
    if dataPresentList[9] == 0:
        try:
            sire5_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire5_link.find("%") >= 0:
                sire5_link = "no_link"

        except TypeError:
            sire5_link = "no_link"
        
    # dam 5
    if dataPresentList[10] == 0:
        try:
            dam5_link = base_url+myAHCAPageParser.get_dam_link()

            if dam5_link.find("%") >= 0:
                dam5_link = "no_link"

        except TypeError:
            dam5_link = "no_link"
            
    # sire 3
    if dataPresentList[5] == 0:
        newKey = build_row(dpw, sire3_link, localDict, 1, keyList[2], 0, keyList, fillKey)

        if fastSearch > 0:
            if key_exists(dbDict, newKey) == 1:
                dataPresentList[11] = dataPresentList[12] = 1

                x = 23
                while x < 27:
                    dataPresentList[x] = 1
                    x += 1

    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    # sire 6
    if dataPresentList[11] == 0:
        try:
            sire6_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire6_link.find("%") >= 0:
                sire6_link = "no_link"

        except TypeError:
            sire6_link = "no_link"
        
    # dam 6
    if dataPresentList[12] == 0:
        try:
            dam6_link = base_url+myAHCAPageParser.get_dam_link()

            if dam6_link.find("%") >= 0:
                dam6_link = "no_link"

        except TypeError:
            dam6_link = "no_link"
            
    
    # dam 3
    if dataPresentList[6] == 0:
        newKey = build_row(dpw, dam3_link, localDict, 1, 0, keyList[2], keyList, fillKey)
    
        if fastSearch > 0:
            if key_exists(dbDict, newKey) == 1:
                dataPresentList[13] = dataPresentList[14] = 1

                x = 27
                while x < 31:
                    dataPresentList[x] = 1
                    x += 1

    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    # sire 7
    if dataPresentList[13] == 0:
        try:
            sire7_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire7_link.find("%") >= 0:
                sire7_link = "no_link"

        except TypeError:
            sire7_link = "no_link"
        
    # dam 7
    if dataPresentList[14] == 0:
        try:
            dam7_link = base_url+myAHCAPageParser.get_dam_link()

            if dam7_link.find("%") >= 0:
                dam7_link = "no_link"

        except TypeError:
            dam7_link = "no_link"
            
    
    # sire 4
    if dataPresentList[7] == 0:
        newKey = build_row(dpw, sire4_link, localDict, 1, keyList[3], 0, keyList, fillKey)
    
        if fastSearch > 0:
            if key_exists(dbDict, newKey) == 1:
                dataPresentList[15] = dataPresentList[16] = 1

    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    # sire 8
    if dataPresentList[15] == 0:
        try:
            sire8_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire8_link.find("%") >= 0:
                sire8_link = "no_link"

        except TypeError:
            sire8_link = "no_link"
        
    # dam 8
    if dataPresentList[16] == 0:
        try:
            dam8_link = base_url+myAHCAPageParser.get_dam_link()

            if dam8_link.find("%") >= 0:
                dam8_link = "no_link"

        except TypeError:
            dam8_link = "no_link"
            
    
    # dam 4
    if dataPresentList[8] == 0:
        newKey = build_row(dpw, dam4_link, localDict, 1, 0, keyList[3], keyList, fillKey)
    
        if fastSearch > 0:
            if key_exists(dbDict, newKey) == 1:
                dataPresentList[17] = dataPresentList[18] = 1

    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    # sire 9
    if dataPresentList[17] == 0:
        try:
            sire9_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire9_link.find("%") >= 0:
                sire9_link = "no_link"

        except TypeError:
            sire9_link = "no_link"
        
    # dam 9
    if dataPresentList[18] == 0:
        try:
            dam9_link = base_url+myAHCAPageParser.get_dam_link()

            if dam9_link.find("%") >= 0:
                dam9_link = "no_link"

        except TypeError:
            dam9_link = "no_link"
            
    
    # sire 5
    if dataPresentList[9] == 0:
        newKey = build_row(dpw, sire5_link, localDict, 1, keyList[4], 0, keyList, fillKey)
    
        if fastSearch > 0:
            if key_exists(dbDict, newKey) == 1:
                dataPresentList[19] = dataPresentList[20] = 1

    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    # sire 10
    if dataPresentList[19] == 0:
        try:
            sire10_link = base_url+myAHCAPageParser.get_sire_link()

            if sire10_link.find("%") >= 0:
                sire10link = "no_link"

        except TypeError:
            sire10_link = "no_link"
        
    # dam 10
    if dataPresentList[20] == 0:
        try:
            dam10_link = base_url+myAHCAPageParser.get_dam_link()

            if dam10_link.find("%") >= 0:
                dam10link = "no_link"

        except TypeError:
            dam10_link = "no_link"
            
    
    # dam 5
    if dataPresentList[10] == 0:
        newKey = build_row(dpw, dam5_link, localDict, 1, 0, keyList[4], keyList, fillKey)
    
        if fastSearch > 0:
            if key_exists(dbDict, newKey) == 1:
                dataPresentList[21] = dataPresentList[22] = 1
    
    
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    # sire 11
    if dataPresentList[21] == 0:
        try:
            sire11_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire11_link.find("%") >= 0:
                sire11_link = "no_link"

        except TypeError:
            sire11_link = "no_link"

    # dam 11
    if dataPresentList[22] == 0:
        try:
            dam11_link = base_url+myAHCAPageParser.get_dam_link()

            if sire11_link.find("%") >= 0:
                sire11_link = "no_link"
        except TypeError:
            dam11_link = "no_link"
        
    # sire 6
    if dataPresentList[11] == 0:
        newKey = build_row(dpw, sire6_link, localDict, 1, keyList[5], 0, keyList, fillKey)
    
        if fastSearch > 0:
            if key_exists(dbDict, newKey) == 1:
                dataPresentList[23] = dataPresentList[24] = 1
        
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    # sire12
    if dataPresentList[23] == 0:
        try:
            sire12_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire12_link.find("%") >= 0:
                sire12_link = "no_link"

        except TypeError:
            sire12_link = "no_link"
        
    # dam 12
    if dataPresentList[24] == 0:
        try:
            dam12_link = base_url+myAHCAPageParser.get_dam_link()

            if dam12_link.find("%") >= 0:
                dam12_link = "no_link"

        except TypeError:
            dam12_link = "no_link"
            
    
    # dam 6
    if dataPresentList[12] == 0:
        newKey = build_row(dpw, dam6_link, localDict, 1, 0, keyList[5], keyList, fillKey)
    
        if fastSearch > 0:
            if key_exists(dbDict, newKey) == 1:
                dataPresentList[25] = dataPresentList[26] = 1
    
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    # sire 13
    if dataPresentList[25] == 0:
        try:
            sire13_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire13_link.find("%") >= 0:
                sire13_link = "no_link"

        except TypeError:
            sire13_link = "no_link"
        
    # dam 13
    if dataPresentList[26] == 0:
        try:
            dam13_link = base_url+myAHCAPageParser.get_dam_link()

            if sire13_link.find("%") >= 0:
                sire13_link = "no_link"

        except TypeError:
            dam13_link = "no_link"
            
    
    # sire 7
    if dataPresentList[13] == 0:
        newKey = build_row(dpw, sire7_link, localDict, 1, keyList[6], 0, keyList, fillKey)
    
        if fastSearch > 0:
            if key_exists(dbDict, newKey) == 1:
                dataPresentList[27] = dataPresentList[28] = 1

    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    # sire 14
    if dataPresentList[27] == 0:
        try:
            sire14_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire14_link.find("%") >= 0:
                sire14_link = "no_link"

        except TypeError:
            sire14_link = "no_link"
        
    # dam 14
    if dataPresentList[28] == 0:
        try:
            dam14_link = base_url+myAHCAPageParser.get_dam_link()

            if dam14_link.find("%") >= 0:
                dam14_link = "no_link"

        except TypeError:
            dam14_link = "no_link"
            
    
    # dam 7
    if dataPresentList[14] == 0:
        newKey = build_row(dpw, dam7_link, localDict, 1, 0, keyList[6], keyList, fillKey)
    
        if fastSearch > 0:
            if key_exists(dbDict, newKey) == 1:
                dataPresentList[29] = dataPresentList[30] = 1
    
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    # sire 15
    if dataPresentList[29] == 0:
        try:
            sire15_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire15_link.find("%") >= 0:
                sire15_link = "no_link"

        except TypeError:
            sire15_link = "no_link"
        
    # dam 15
    if dataPresentList[30] == 0:
        try:
            dam15_link = base_url+myAHCAPageParser.get_dam_link()

            if dam15_link.find("%") >= 0:
                dam15_link = "no_link"

        except TypeError:
            dam15_link = "no_link"
            
    
    # sire 8
    if dataPresentList[15] == 0:
        newKey = build_row(dpw, sire8_link, localDict, 1, keyList[7], 0, keyList, fillKey)
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    if dataPresentList[15] == 0:
        try:
            sire16_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire16_link.find("%") >= 0:
                sire16_link = "no_link"

        except TypeError:
            sire16_link = "no_link"
        
        try:
            dam16_link = base_url+myAHCAPageParser.get_dam_link()

            if dam16_link.find("%") >= 0:
                dam16_link = "no_link"

        except TypeError:
            dam16_link = "no_link"
            
    
    # dam 8
    if dataPresentList[16] == 0:
        newKey = build_row(dpw, dam8_link, localDict, 1, 0, keyList[7], keyList, fillKey)
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    if dataPresentList[16] == 0:
        try:
            sire17_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire17_link.find("%") >= 0:
                sire17_link = "no_link"

        except TypeError:
            sire17_link = "no_link"
        
        try:
            dam17_link = base_url+myAHCAPageParser.get_dam_link()

            if dam17_link.find("%") >= 0:
                dam17_link = "no_link"

        except TypeError:
            dam17_link = "no_link"
            
    
    # sire 9
    if dataPresentList[17] == 0:
        newKey = build_row(dpw, sire9_link, localDict, 1, keyList[8], 0, keyList, fillKey)
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    if dataPresentList[17] == 0:
        try:
            sire18_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire18_link.find("%") >= 0:
                sire18_link = "no_link"

        except TypeError:
            sire18_link = "no_link"
        
        try:
            dam18_link = base_url+myAHCAPageParser.get_dam_link()

            if dam18_link.find("%") >= 0:
                dam18_link = "no_link"

        except TypeError:
            dam18_link = "no_link"
            
    
    # dam 9
    if dataPresentList[18] == 0:
        newKey = build_row(dpw, dam9_link, localDict, 1, 0, keyList[8], keyList, fillKey)
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    if dataPresentList[18] == 0:
        try:
            sire19_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire19_link.find("%") >= 0:
                sire19_link = "no_link"

        except TypeError:
            sire19_link = "no_link"
        
        try:
            dam19_link = base_url+myAHCAPageParser.get_dam_link()

            if dam19_link.find("%") >= 0:
                dam19_link = "no_link"

        except TypeError:
            dam19_link = "no_link"
            
    
    # sire 10
    if dataPresentList[19] == 0:
        newKey = build_row(dpw, sire10_link, localDict, 1, keyList[9], 0, keyList, fillKey)
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    if dataPresentList[19] == 0:
        try:
            sire20_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire20_link.find("%") >= 0:
                sire20_link = "no_link"

        except TypeError:
            sire20_link = "no_link"
        
        try:
            dam20_link = base_url+myAHCAPageParser.get_dam_link()

            if dam20_link.find("%") >= 0:
                dam20_link = "no_link"

        except TypeError:
            dam20_link = "no_link"
            
    
    # dam 10
    if dataPresentList[20] == 0:
        newKey = build_row(dpw, dam10_link, localDict, 1, 0, keyList[9], keyList, fillKey)
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    if dataPresentList[20] == 0:
        try:
            sire21_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire21_link.find("%") >= 0:
                sire21_link = "no_link"

        except TypeError:
            sire21_link = "no_link"
        
        try:
            dam21_link = base_url+myAHCAPageParser.get_dam_link()

            if dam21_link.find("%") >= 0:
                dam21_link = "no_link"

        except TypeError:
            dam21_link = "no_link"
            
    
    # sire 11
    if dataPresentList[21] == 0:
        newKey = build_row(dpw, sire11_link, localDict, 1, keyList[10], 0, keyList, fillKey)
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    if dataPresentList[21] == 0:
        try:
            sire22_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire22_link.find("%") >= 0:
                sire22_link = "no_link"

        except TypeError:
            sire22_link = "no_link"
        
        try:
            dam22_link = base_url+myAHCAPageParser.get_dam_link()

            if dam22_link.find("%") >= 0:
                dam22_link = "no_link"

        except TypeError:
            dam22_link = "no_link"
            
    
    # dam 11
    if dataPresentList[22] == 0:
        newKey = build_row(dpw, dam11_link, localDict, 1, 0, keyList[10], keyList, fillKey)
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    if dataPresentList[22] == 0:
        try:
            sire23_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire23_link.find("%") >= 0:
                sire23_link = "no_link"

        except TypeError:
            sire23_link = "no_link"
        
        try:
            dam23_link = base_url+myAHCAPageParser.get_dam_link()

            if dam23_link.find("%") >= 0:
                dam23_link = "no_link"

        except TypeError:
            dam23_link = "no_link"
            
    
    # sire12
    if dataPresentList[23] == 0:
        newKey = build_row(dpw, sire12_link, localDict, 1, keyList[11], 0, keyList, fillKey)
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    if dataPresentList[23] == 0:
        try:
            sire24_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire24_link.find("%") >= 0:
                sire24_link = "no_link"

        except TypeError:
            sire24_link = "no_link"
        
        try:
            dam24_link = base_url+myAHCAPageParser.get_dam_link()

            if dam24_link.find("%") >= 0:
                dam24_link = "no_link"

        except TypeError:
            dam24_link = "no_link"
            
    
    # dam 12
    if dataPresentList[24] == 0:
        newKey = build_row(dpw, dam12_link, localDict, 1, 0, keyList[11], keyList, fillKey)
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    if dataPresentList[24] == 0:
        try:
            sire25_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire25_link.find("%") >= 0:
                sire25_link = "no_link"

        except TypeError:
            sire25_link = "no_link"
        
        try:
            dam25_link = base_url+myAHCAPageParser.get_dam_link()

            if dam25_link.find("%") >= 0:
                dam25_link = "no_link"

        except TypeError:
            dam25_link = "no_link"
            
    
    # sire 13
    if dataPresentList[25] == 0:
        newKey = build_row(dpw, sire13_link, localDict, 1, keyList[12], 0, keyList, fillKey)
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    if dataPresentList[25] == 0:
        try:
            sire26_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire26_link.find("%") >= 0:
                sire26_link = "no_link"

        except TypeError:
            sire26_link = "no_link"
        
        try:
            dam26_link = base_url+myAHCAPageParser.get_dam_link()

            if dam26_link.find("%") >= 0:
                dam26_link = "no_link"

        except TypeError:
            dam26_link = "no_link"
            
    
    # dam 13
    if dataPresentList[26] == 0:
        newKey = build_row(dpw, dam13_link, localDict, 1, 0, keyList[12], keyList, fillKey)
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    if dataPresentList[26] == 0:
        try:
            sire27_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire27_link.find("%") >= 0:
                sire27_link = "no_link"

        except TypeError:
            sire27_link = "no_link"
        
        try:
            dam27_link = base_url+myAHCAPageParser.get_dam_link()

            if dam27_link.find("%") >= 0:
                dam27_link = "no_link"

        except TypeError:
            dam27_link = "no_link"
            
    
    # sire 14
    if dataPresentList[27] == 0:
        newKey = build_row(dpw, sire14_link, localDict, 1, keyList[13], 0, keyList, fillKey)
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    if dataPresentList[27] == 0:
        try:
            sire28_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire28_link.find("%") >= 0:
                sire28_link = "no_link"

        except TypeError:
            sire28_link = "no_link"
        
        try:
            dam28_link = base_url+myAHCAPageParser.get_dam_link()

            if dam28_link.find("%") >= 0:
                dam28_link = "no_link"

        except TypeError:
            dam28_link = "no_link"
            
    
    # dam 14
    if dataPresentList[28] == 0:
        newKey = build_row(dpw, dam14_link, localDict, 1, 0, keyList[13], keyList, fillKey)
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    if dataPresentList[28] == 0:
        try:
            sire29_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire29_link.find("%") >= 0:
                sire29_link = "no_link"

        except TypeError:
            sire29_link = "no_link"
        
        try:
            dam29_link = base_url+myAHCAPageParser.get_dam_link()

            if dam29_link.find("%") >= 0:
                dam29_link = "no_link"

        except TypeError:
            dam29_link = "no_link"
            
    
    # sire 15
    if dataPresentList[29] == 0:
        newKey = build_row(dpw, sire15_link, localDict, 1, keyList[14], 0, keyList, fillKey)
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    if dataPresentList[29] == 0:
        try:
            sire30_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire30_link.find("%") >= 0:
                sire30_link = "no_link"

        except TypeError:
            sire30_link = "no_link"
        
        try:
            dam30_link = base_url+myAHCAPageParser.get_dam_link()

            if dam30_link.find("%") >= 0:
                dam30_link = "no_link"

        except TypeError:
            dam30_link = "no_link"
            
    
    # dam 15
    if dataPresentList[30] == 0:
        newKey = build_row(dpw, dam15_link, localDict, 1, 0, keyList[14], keyList, fillKey)
    else:
        raw_row[0] = fillKey
        localDict[fillKey] = raw_row
        keyList.append(newKey)

    fillKey = fillKey + 1
    
    if dataPresentList[30] == 0:
        try:
            sire31_link =  base_url+myAHCAPageParser.get_sire_link()

            if sire31_link.find("%") >= 0:
                sire31_link = "no_link"

        except TypeError:
            sire31_link = "no_link"
        
        try:
            dam31_link = base_url+myAHCAPageParser.get_dam_link()

            if dam31_link.find("%") >= 0:
                dam31_link = "no_link"

        except TypeError:
            dam31_link = "no_link"
            

    # sire 16
    if dataPresentList[15] == 0:
        newKey = build_row(dpw, sire16_link, localDict, 0, keyList[15], 0, keyList, fillKey)
    
        # dam 16
        newKey = build_row(dpw, dam16_link, localDict, 0, 0, keyList[15], keyList, fillKey)

    # sire 17
    if dataPresentList[16] == 0:
        newKey = build_row(dpw, sire17_link, localDict, 0, keyList[16], 0, keyList, fillKey)
    
        # dam 17
        newKey = build_row(dpw, dam17_link, localDict, 0, 0, keyList[16], keyList, fillKey)

    # sire 18
    if dataPresentList[17] == 0:
        newKey = build_row(dpw, sire18_link, localDict, 0, keyList[17], 0, keyList, fillKey)
    
        # dam 18
        newKey = build_row(dpw, dam18_link, localDict, 0, 0, keyList[17], keyList, fillKey)

    # sire 19
    if dataPresentList[18] == 0:
        newKey = build_row(dpw, sire19_link, localDict, 0, keyList[18], 0, keyList, fillKey)
    
        # dam 19
        newKey = build_row(dpw, dam19_link, localDict, 0, 0, keyList[18], keyList, fillKey)

    # sire 20
    if dataPresentList[19] == 0:
        newKey = build_row(dpw, sire20_link, localDict, 0, keyList[19], 0, keyList, fillKey)
    
        # dam 20
        newKey = build_row(dpw, dam20_link, localDict, 0, 0, keyList[19], keyList, fillKey)

    # sire 21
    if dataPresentList[20] == 0:
        newKey = build_row(dpw, sire21_link, localDict, 0, keyList[20], 0, keyList, fillKey)
    
        # dam 21
        newKey = build_row(dpw, dam21_link, localDict, 0, 0, keyList[20], keyList, fillKey)

    # sire 22
    if dataPresentList[21] == 0:
        newKey = build_row(dpw, sire22_link, localDict, 0, keyList[21], 0, keyList, fillKey)
    
        # dam 22
        newKey = build_row(dpw, dam22_link, localDict, 0, 0, keyList[21], keyList, fillKey)

    # sire 23
    if dataPresentList[22] == 0:
        newKey = build_row(dpw, sire23_link, localDict, 0, keyList[22], 0, keyList, fillKey)
    
        # dam 23
        newKey = build_row(dpw, dam23_link, localDict, 0, 0, keyList[22], keyList, fillKey)

    # sire 24
    if dataPresentList[23] == 0:
        newKey = build_row(dpw, sire24_link, localDict, 0, keyList[23], 0, keyList, fillKey)
    
        # dam 24
        newKey = build_row(dpw, dam24_link, localDict, 0, 0, keyList[23], keyList, fillKey)

    # sire 25
    if dataPresentList[24] == 0:
        newKey = build_row(dpw, sire25_link, localDict, 0, keyList[24], 0, keyList, fillKey)
    
        # dam 25
        newKey = build_row(dpw, dam25_link, localDict, 0, 0, keyList[24], keyList, fillKey)

    # sire 26
    if dataPresentList[25] == 0:
        newKey = build_row(dpw, sire26_link, localDict, 0, keyList[25], 0, keyList, fillKey)
    
        # dam 26
        newKey = build_row(dpw, dam26_link, localDict, 0, 0, keyList[25], keyList, fillKey)

    # sire 27
    if dataPresentList[26] == 0:
        newKey = build_row(dpw, sire27_link, localDict, 0, keyList[26], 0, keyList, fillKey)
    
        # dam 27
        newKey = build_row(dpw, dam27_link, localDict, 0, 0, keyList[26], keyList, fillKey)

    # sire 28
    if dataPresentList[27] == 0:
        newKey = build_row(dpw, sire28_link, localDict, 0, keyList[27], 0, keyList, fillKey)
    
        # dam 28
        newKey = build_row(dpw, dam28_link, localDict, 0, 0, keyList[27], keyList, fillKey)

    # sire 29
    if dataPresentList[28] == 0:
        newKey = build_row(dpw, sire29_link, localDict, 0, keyList[28], 0, keyList, fillKey)
    
        # dam 29
        newKey = build_row(dpw, dam29_link, localDict, 0, 0, keyList[28], keyList, fillKey)

    # sire 30
    if dataPresentList[29] == 0:
        newKey = build_row(dpw, sire30_link, localDict, 0, keyList[29], 0, keyList, fillKey)
    
        # dam 30
        newKey = build_row(dpw, dam30_link, localDict, 0, 0, keyList[29], keyList, fillKey)

    # sire 31
    if dataPresentList[30] == 0:
        newKey = build_row(dpw, sire31_link, localDict, 0, keyList[30] , 0, keyList, fillKey)
    
        # dam 31
        newKey = build_row(dpw, dam31_link, localDict, 0, 0, keyList[30], keyList, fillKey)


    # Now, update the master database
    # Open it up for updating
    dbFile = open(dataBaseFile, 'a+')
    
    if dpw != None:
        # Create a display window
        dpw2 = displayDbUpdate()
    else:
        dpw2 = None

    # For each entry in the new animal's dictionary, check to see
    # if it is already in the master database.
    localKeys = localDict.keys()
    
    for x in range(localDict.__len__()):
        try:
            duplicateEntry = 0

            key = localKeys[x]

            if localDict[key][2] != '':
                
                for y in range(dbDict.__len__()):
                    try:
                        if key == masterKeys[y]:

                            # The names don't always match exactly. Sometimes
                            # They include the foreign registration number
                            # in parenthesis, sometimes they don't. We'll
                            # force the entry for the new animal to the 
                            # entry we currently have for it.
                            if dbDict[key][2].split(" (")[0] == localDict[key][2].split(" (")[0]:
                                if dpw == None:
                                    print localDict[key][2], " already exists as ", dbDict[key][2]
                                else:
                                    displayDbUpdate.addLine(dpw2, localDict[key][2] + " already exists as " + dbDict[key][2])

                                localDict[key][2] = dbDict[key][2]
                                duplicateEntry = 1
 
                            break
                
                    except IndexError:
                        break
            
                if duplicateEntry == 0:
                    # We have a new entry, add it to the database
                    l = localDict[key]
                    if dpw == None:
                        print "Adding animal to database: ", l[2]
                    else:
                        displayDbUpdate.addLine(dpw2, "Adding animal to database: " + l[2])


                    row = ['"%s"' % i for i in l ]
                    print >> dbFile, ",".join(row)
            
        except IndexError:
            break

    # All done!

    dbFile.close()

    if dpw == None:
        print dataBaseFile, " has been updated."
    else:
        tkMessageBox.showinfo("Update Complete", dataBaseFile + " has been updated.")

    sys.exit(0)
    
